'''
    Simple socket server using threads
'''
 
import logging
import socket
import sys
import numbers
import signal
from thread import *
from datetime import datetime
from spi_if import *
import oled

# Set LOGGING_ENABLED = 0 to disable printing to the log file.
LOGGING_ENABLED = 1
LOGGING_FILENAME = '/repo/k7168/k7168_server.log'
logging.basicConfig(filename=LOGGING_FILENAME, format='%(asctime)s %(levelname)s:%(message)s',level=logging.INFO)
 
exttemp_str = ''
rh_str = ''
hecto_str = ''

HOST = ''   # Symbolic name meaning all available interfaces
PORT = 10001 # Arbitrary non-privileged port
 
s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

oled = oled.device()
relay_symbol = [0xee, 0xef]  # open and close symbols for relay icon
lcd_info_str = '                '

# Channel status, 0 when open, 1 when closed
scan_ch = [0, 0, 0, 0, 0, 0, 0, 0] 

oled.initialize() 
#time.sleep(0.5)
oled.write_disp ("xDevs.com X7168",0)  # Write message on OLED line 0
oled.write_disp ("%c %c %c %c %c %c %c %cL" % (relay_symbol[scan_ch[7]], relay_symbol[scan_ch[6]], relay_symbol[scan_ch[5]], relay_symbol[scan_ch[4]], relay_symbol[scan_ch[3]], relay_symbol[scan_ch[2]], relay_symbol[scan_ch[1]], relay_symbol[scan_ch[0]] ), 1)  # Write message on OLED line 0
    
if (LOGGING_ENABLED):
  logging.info('Socket created')
 
#Bind socket to local host and port
try:
    s.bind((HOST, PORT))
except socket.error as msg:
    logging.error('Bind failed. Error Code : ' + str(msg[0]) + ' Message ' + msg[1])
    lcd_info_str = 'Socket error!    '
    oled.write_disp ("%s" % lcd_info_str, 0)  # Write message on OLED line 0
    sys.exit()
     
if (LOGGING_ENABLED):
  logging.info('Socket bind complete')
 
#Start listening on socket
s.listen(10)


if (LOGGING_ENABLED):
  logging.info('Socket now listening')
 
#Function for handling connections. This will be used to create threads
def clientthread(conn,ipaddr,pid):
    #Sending message to connected client
    #conn.send('Welcome to the server. Type something and hit enter\n') #send only takes string
    #infinite loop so that function do not terminate and thread do not end.
    while True:

        global scan_ch 

        #Receiving from client
        data = conn.recv(1024)
        #Remove carriage return and line feed
        data = data.strip('\n')
        data = data.strip('\r')
        data = data.upper()
        lcd_info_str = 'Data received   '
        if(data == '*IDN?'):
                #get_THP()
                reply = 'xDevs.com Keithley 7176 nV-scanner,Rev.0,BETA' #exttemp_str + ',' + rh_str
                conn.sendall(reply)
                conn.sendall('\r')
        elif (data == '*RST'):
                print "Resetting"
                lcd_info_str = 'All open         '
                scan_ch = [0,0,0,0, 0,0,0,0]
                reply = "System reset"
                conn.sendall(reply)
                conn.sendall('\r')
        elif (data == ':CLOS?'):
                print "Report closed channels"
                reply = "%s" % scan_ch
                conn.sendall(reply)
                conn.sendall('\r')
        elif (data[0:5] == ':CLOS'):
                if (data[7] == "!"):
                    # Close channel
                    sel_ch = int(data[6]) - 1
		    if (sel_ch > 7):
			lcd_info_str = 'Incorrect ch CLS'
                        reply = "253"
		    else:
                        scan_ch[sel_ch] = 1
	                lcd_info_str = 'Channel %d close' % scan_ch[sel_ch]
    	                reply = "%s" % scan_ch
                else:
                    lcd_info_str = 'Syntax error CLS'
                    reply = "255"
                conn.sendall(reply)
                conn.sendall('\r')
        elif (data[0:5] == ':OPEN'):
                if (data[7] == "!"):
                    # Open channel 
                    sel_ch = int(data[6]) - 1
		    if (sel_ch > 7):
			lcd_info_str = 'Incorrect ch CLS'
                        reply = "252"
		    else:
                        scan_ch[sel_ch] = 0
	                lcd_info_str = 'Channel %d open ' % scan_ch[sel_ch]
    	                reply = "%s" % scan_ch
                else:
                    lcd_info_str = 'Syntax error OPN'
                    reply = "254"
                conn.sendall(reply)
                conn.sendall('\r')
        elif (data == ':SYST:ERR?'):
                print "Check errors here"
                reply = "255"
                conn.sendall(reply)
                conn.sendall('\r')
        elif (data == 'READ?3'):
                get_THP()
                reply = exttemp_str + ',' + rh_str + ',' + hecto_str
                conn.sendall(reply)
                conn.sendall('\r')
        #else :
                #reply = 'Unknown command ' + data
                #conn.sendall(reply)
                #conn.sendall('\n')
        if not data: 
            break
        spi_write(scan_ch[7] << 7 | scan_ch[6] << 6 | scan_ch[5] << 5 | scan_ch[4] << 4 | scan_ch[3] << 3 | scan_ch[2] << 2 | scan_ch[1] << 1 | scan_ch[0])
        oled.write_disp ("%s" % lcd_info_str, 0)  # Write message on OLED line 0
        oled.write_disp ("%c %c %c %c %c %c %c %cL" % (relay_symbol[scan_ch[7]], relay_symbol[scan_ch[6]], relay_symbol[scan_ch[5]], relay_symbol[scan_ch[4]], relay_symbol[scan_ch[3]], relay_symbol[scan_ch[2]], relay_symbol[scan_ch[1]], relay_symbol[scan_ch[0]] ), 1)  # Write message on OLED line 0

    if (LOGGING_ENABLED):    
      logging.info('Disconnected from ' + ipaddr + ' : ' + pid) 
    #came out of loop
    conn.close()

# Read Temperature, Humidity, and Barometric Pressure from BME280
def get_THP():
    global exttemp
    global rh
    global pascals
    global hectopascals
    global exttemp_str
    global rh_str
    global hecto_str

    exttemp = 1 #sensor.read_temperature()
    rh = 2# sensor.read_humidity()
    pascals = 3# sensor.read_pressure()
    hectopascals = 4# pascals / 100

    exttemp_str = '{0:.2f}'.format(exttemp)
    rh_str = '{0:.1f}'.format(rh)
    hecto_str = '{0:.3f}'.format(hectopascals)
 
#now keep talking with the client
while 1:
    #wait to accept a connection - blocking call
    conn, addr = s.accept()
    if (LOGGING_ENABLED):
      logging.info('Connected with ' + addr[0] + ' : ' + str(addr[1]))
     
    #start new thread takes 1st argument as a function name to be run, second is the tuple of arguments to the function.
    start_new_thread (clientthread ,(conn,str(addr[0]),str(addr[1]),))
 
s.close()
